@@ -34,11 +34,11 @@ from PyQt5.QtWebEngineWidgets import QWebEngineSettings, QWebEngineProfile
 
 from qutebrowser.browser import history
 from qutebrowser.browser.webengine import (spell, webenginequtescheme, cookies,
-                                           webenginedownloads)
+                                           webenginedownloads, notification)
 from qutebrowser.config import config, websettings
 from qutebrowser.config.websettings import AttributeInfo as Attr
 from qutebrowser.utils import (standarddir, qtutils, message, log,
-                               urlmatch, usertypes, objreg)
+                               urlmatch, usertypes, objreg, version)
 if TYPE_CHECKING:
     from qutebrowser.browser.webengine import interceptor
 
@@ -360,6 +360,9 @@ def _init_profile(profile: QWebEngineProfile) -> None:
     _download_manager.install(profile)
     cookies.install_filter(profile)
 
+    if notification.bridge is not None:
+        notification.bridge.install(profile)
+
     # Clear visited links on web history clear
     history.web_history.history_cleared.connect(profile.clearAllVisitedLinks)
     history.web_history.url_cleared.connect(
@@ -374,7 +377,17 @@ def _init_default_profile():
 
     default_profile = QWebEngineProfile.defaultProfile()
 
+    assert parsed_user_agent is None  # avoid earlier profile initialization
+    non_ua_version = version.qtwebengine_versions(avoid_init=True)
+
     init_user_agent()
+    ua_version = version.qtwebengine_versions()
+    if ua_version.webengine != non_ua_version.webengine:
+        log.init.warning(
+            "QtWebEngine version mismatch - unexpected behavior might occur, "
+            "please open a bug about this.\n"
+            f"  Early version: {non_ua_version}\n"
+            f"  Real version:  {ua_version}")
 
     default_profile.setCachePath(
         os.path.join(standarddir.cache(), 'webengine'))
@@ -401,7 +414,7 @@ def _init_site_specific_quirks():
 
     See https://github.com/qutebrowser/qutebrowser/issues/4810
     """
-    if not config.val.content.site_specific_quirks:
+    if not config.val.content.site_specific_quirks.enabled:
         return
 
     # Please leave this here as a template for new UAs.
@@ -418,35 +431,40 @@ def _init_site_specific_quirks():
                      "AppleWebKit/537.36 (KHTML, like Gecko) "
                      "Chrome/99 "
                      "Safari/537.36")
-    edge_ua = ("Mozilla/5.0 ({os_info}) "
-               "AppleWebKit/{webkit_version} (KHTML, like Gecko) "
-               "{upstream_browser_key}/{upstream_browser_version} "
-               "Safari/{webkit_version} "
-               "Edg/{upstream_browser_version}")
+    firefox_ua = "Mozilla/5.0 ({os_info}; rv:90.0) Gecko/20100101 Firefox/90.0"
 
-    user_agents = {
+    user_agents = [
         # Needed to avoid a ""WhatsApp works with Google Chrome 36+" error
         # page which doesn't allow to use WhatsApp Web at all. Also see the
         # additional JS quirk: qutebrowser/javascript/quirks/whatsapp_web.user.js
         # https://github.com/qutebrowser/qutebrowser/issues/4445
-        'https://web.whatsapp.com/': no_qtwe_ua,
+        ("ua-whatsapp", 'https://web.whatsapp.com/', no_qtwe_ua),
 
         # Needed to avoid a "you're using a browser [...] that doesn't allow us
         # to keep your account secure" error.
         # https://github.com/qutebrowser/qutebrowser/issues/5182
-        'https://accounts.google.com/*': edge_ua,
+        ("ua-google", 'https://accounts.google.com/*', firefox_ua),
 
         # Needed because Slack adds an error which prevents using it relatively
         # aggressively, despite things actually working fine.
         # September 2020: Qt 5.12 works, but Qt <= 5.11 shows the error.
         # https://github.com/qutebrowser/qutebrowser/issues/4669
-        'https://*.slack.com/*': new_chrome_ua,
-    }
+        ("ua-slack", 'https://*.slack.com/*', new_chrome_ua),
+    ]
 
-    for pattern, ua in user_agents.items():
-        config.instance.set_obj('content.headers.user_agent', ua,
-                                pattern=urlmatch.UrlPattern(pattern),
-                                hide_userconfig=True)
+    for name, pattern, ua in user_agents:
+        if name not in config.val.content.site_specific_quirks.skip:
+            config.instance.set_obj('content.headers.user_agent', ua,
+                                    pattern=urlmatch.UrlPattern(pattern),
+                                    hide_userconfig=True)
+
+    if 'misc-krunker' not in config.val.content.site_specific_quirks.skip:
+        config.instance.set_obj(
+            'content.headers.accept_language',
+            '',
+            pattern=urlmatch.UrlPattern('https://matchmaker.krunker.io/*'),
+            hide_userconfig=True,
+        )
 
 
 def _init_devtools_settings():
@@ -489,13 +507,19 @@ def init():
     from qutebrowser.misc import quitter
     quitter.instance.shutting_down.connect(_download_manager.shutdown)
 
+    log.init.debug("Initializing notification presenter...")
+    notification.init()
+
+    log.init.debug("Initializing global settings...")
     global _global_settings
     _global_settings = WebEngineSettings(_SettingsWrapper())
 
+    log.init.debug("Initializing profiles...")
     _init_default_profile()
     init_private_profile()
     config.instance.changed.connect(_update_settings)
 
+    log.init.debug("Misc initialization...")
     _init_site_specific_quirks()
     _init_devtools_settings()
 
