@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,7 +15,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Completion item delegate for CompletionView.
 
@@ -26,15 +26,39 @@ import re
 import html
 
 from PyQt5.QtWidgets import QStyle, QStyleOptionViewItem, QStyledItemDelegate
-from PyQt5.QtCore import QRectF, QSize, Qt
+from PyQt5.QtCore import QRectF, QRegularExpression, QSize, Qt
 from PyQt5.QtGui import (QIcon, QPalette, QTextDocument, QTextOption,
-                         QAbstractTextDocumentLayout)
+                         QAbstractTextDocumentLayout, QSyntaxHighlighter,
+                         QTextCharFormat)
 
 from qutebrowser.config import config
-from qutebrowser.utils import qtutils, jinja
+from qutebrowser.utils import qtutils
 
 
-_cached_stylesheet = None
+class _Highlighter(QSyntaxHighlighter):
+
+    def __init__(self, doc, pattern, color):
+        super().__init__(doc)
+        self._format = QTextCharFormat()
+        self._format.setForeground(color)
+        words = pattern.split()
+        words.sort(key=len, reverse=True)
+        pat = "|".join(re.escape(word) for word in words)
+        self._expression = QRegularExpression(
+            pat, QRegularExpression.CaseInsensitiveOption
+        )
+        qtutils.ensure_valid(self._expression)
+
+    def highlightBlock(self, text):
+        """Override highlightBlock for custom highlighting."""
+        match_iterator = self._expression.globalMatch(text)
+        while match_iterator.hasNext():
+            match = match_iterator.next()
+            self.setFormat(
+                match.capturedStart(),
+                match.capturedLength(),
+                self._format
+            )
 
 
 class CompletionItemDelegate(QStyledItemDelegate):
@@ -68,11 +92,16 @@ class CompletionItemDelegate(QStyledItemDelegate):
 
     def _draw_background(self):
         """Draw the background of an ItemViewItem."""
+        assert self._opt is not None
+        assert self._style is not None
         self._style.drawPrimitive(self._style.PE_PanelItemViewItem, self._opt,
                                   self._painter, self._opt.widget)
 
     def _draw_icon(self):
         """Draw the icon of an ItemViewItem."""
+        assert self._opt is not None
+        assert self._style is not None
+
         icon_rect = self._style.subElementRect(
             self._style.SE_ItemViewItemDecoration, self._opt, self._opt.widget)
         if not icon_rect.isValid():
@@ -98,6 +127,9 @@ class CompletionItemDelegate(QStyledItemDelegate):
         Args:
             index: The QModelIndex of the item to draw.
         """
+        assert self._opt is not None
+        assert self._painter is not None
+        assert self._style is not None
         if not self._opt.text:
             return
 
@@ -147,16 +179,20 @@ class CompletionItemDelegate(QStyledItemDelegate):
         Args:
             rect: The QRect to clip the drawing to.
         """
+        assert self._painter is not None
+        assert self._doc is not None
+        assert self._opt is not None
+
         # We can't use drawContents because then the color would be ignored.
         clip = QRectF(0, 0, rect.width(), rect.height())
         self._painter.save()
 
         if self._opt.state & QStyle.State_Selected:
-            color = config.val.colors.completion.item.selected.fg
+            color = config.cache['colors.completion.item.selected.fg']
         elif not self._opt.state & QStyle.State_Enabled:
-            color = config.val.colors.completion.category.fg
+            color = config.cache['colors.completion.category.fg']
         else:
-            colors = config.val.colors.completion.fg
+            colors = config.cache['colors.completion.fg']
             # if multiple colors are set, use different colors per column
             color = colors[col % len(colors)]
         self._painter.setPen(color)
@@ -175,6 +211,7 @@ class CompletionItemDelegate(QStyledItemDelegate):
         Args:
             index: The QModelIndex of the item to draw.
         """
+        assert self._opt is not None
         # FIXME we probably should do eliding here. See
         # qcommonstyle.cpp:viewItemDrawText
         # https://github.com/qutebrowser/qutebrowser/issues/118
@@ -194,21 +231,17 @@ class CompletionItemDelegate(QStyledItemDelegate):
         self._doc.setDefaultTextOption(text_option)
         self._doc.setDocumentMargin(2)
 
-        assert _cached_stylesheet is not None
-        self._doc.setDefaultStyleSheet(_cached_stylesheet)
-
         if index.parent().isValid():
             view = self.parent()
             pattern = view.pattern
             columns_to_filter = index.model().columns_to_filter(index)
             if index.column() in columns_to_filter and pattern:
-                repl = r'<span class="highlight">\g<0></span>'
-                text = re.sub(re.escape(pattern).replace(r'\ ', r'|'),
-                              repl, html.escape(self._opt.text),
-                              flags=re.IGNORECASE)
-                self._doc.setHtml(text)
-            else:
-                self._doc.setPlainText(self._opt.text)
+                if self._opt.state & QStyle.State_Selected:
+                    color = config.val.colors.completion.item.selected.match.fg
+                else:
+                    color = config.val.colors.completion.match.fg
+                _Highlighter(self._doc, pattern, color)
+            self._doc.setPlainText(self._opt.text)
         else:
             self._doc.setHtml(
                 '<span style="font: {};">{}</span>'.format(
@@ -217,13 +250,15 @@ class CompletionItemDelegate(QStyledItemDelegate):
 
     def _draw_focus_rect(self):
         """Draw the focus rectangle of an ItemViewItem."""
+        assert self._opt is not None
+        assert self._style is not None
         state = self._opt.state
         if not state & QStyle.State_HasFocus:
             return
         o = self._opt
         o.rect = self._style.subElementRect(
             self._style.SE_ItemViewItemFocusRect, self._opt, self._opt.widget)
-        o.state |= QStyle.State_KeyboardFocusChange | QStyle.State_Item
+        o.state |= int(QStyle.State_KeyboardFocusChange | QStyle.State_Item)
         qtutils.ensure_valid(o.rect)
         if state & QStyle.State_Enabled:
             cg = QPalette.Normal
@@ -256,12 +291,15 @@ class CompletionItemDelegate(QStyledItemDelegate):
         self._opt = QStyleOptionViewItem(option)
         self.initStyleOption(self._opt, index)
         self._style = self._opt.widget.style()
+
         self._get_textdoc(index)
+        assert self._doc is not None
+
         docsize = self._doc.size().toSize()
         size = self._style.sizeFromContents(QStyle.CT_ItemViewItem, self._opt,
                                             docsize, self._opt.widget)
         qtutils.ensure_valid(size)
-        return size + QSize(10, 3)
+        return size + QSize(10, 3)  # type: ignore[operator]
 
     def paint(self, painter, option, index):
         """Override the QStyledItemDelegate paint function.
@@ -283,24 +321,3 @@ class CompletionItemDelegate(QStyledItemDelegate):
         self._draw_focus_rect()
 
         self._painter.restore()
-
-
-@config.change_filter('colors.completion.match.fg', function=True)
-def _update_stylesheet():
-    """Update the cached stylesheet."""
-    stylesheet = """
-        .highlight {
-            color: {{ conf.colors.completion.match.fg }};
-        }
-    """
-    with jinja.environment.no_autoescape():
-        template = jinja.environment.from_string(stylesheet)
-
-    global _cached_stylesheet
-    _cached_stylesheet = template.render(conf=config.val)
-
-
-def init():
-    """Initialize the cached stylesheet."""
-    _update_stylesheet()
-    config.instance.changed.connect(_update_stylesheet)
