@@ -23,7 +23,9 @@ import io
 import logging
 import pathlib
 import functools
-from typing import Optional, IO
+import contextlib
+import subprocess
+from typing import Optional, IO, Iterator
 
 from PyQt5.QtCore import QUrl
 
@@ -116,6 +118,37 @@ def _resource_type_to_string(resource_type: Optional[ResourceType]) -> str:
     return _RESOURCE_TYPE_STRINGS.get(resource_type, "other")
 
 
+class DeserializationError(Exception):
+
+    """Custom exception for adblock.DeserializationErrors.
+
+    See _map_exception below for details.
+    """
+
+
+@contextlib.contextmanager
+def _map_exceptions() -> Iterator[None]:
+    """Handle exception API differences in adblock 0.5.0.
+
+    adblock < 0.5.0 will raise a ValueError with a string describing the
+    exception class for all exceptions. With adblock 0.5.0+, it raises proper
+    exception classes.
+
+    This context manager unifies the two (only for DeserializationError so far).
+    """
+    adblock_deserialization_error = getattr(
+        adblock, "DeserializationError", ValueError)
+
+    try:
+        yield
+    except adblock_deserialization_error as e:
+        if isinstance(e, ValueError) and str(e) != "DeserializationError":
+            # All Rust exceptions get turned into a ValueError by
+            # python-adblock
+            raise
+        raise DeserializationError(str(e))
+
+
 class BraveAdBlocker:
 
     """Manage blocked hosts based on Brave's adblocker.
@@ -131,7 +164,23 @@ class BraveAdBlocker:
         self.enabled = _should_be_used()
         self._has_basedir = has_basedir
         self._cache_path = data_dir / "adblock-cache.dat"
-        self._engine = adblock.Engine(adblock.FilterSet())
+        try:
+            self._engine = adblock.Engine(adblock.FilterSet())
+        except AttributeError:
+            # this should never happen - let's get some infos if it does
+            logger.debug(f"adblock module: {adblock}")
+            dist = version.distribution()
+            if (dist is not None and
+                    dist.parsed == version.Distribution.arch and
+                    hasattr(adblock, "__file__")):
+                proc = subprocess.run(
+                    ['pacman', '-Qo', adblock.__file__],
+                    stdout=subprocess.PIPE,
+                    universal_newlines=True,
+                    check=False,
+                )
+                logger.debug(proc.stdout)
+            raise
 
     def _is_blocked(
         self,
@@ -203,17 +252,27 @@ class BraveAdBlocker:
 
     def read_cache(self) -> None:
         """Initialize the adblocking engine from cache file."""
-        if self._cache_path.is_file():
+        try:
+            cache_exists = self._cache_path.is_file()
+        except OSError:
+            logger.error("Failed to read adblock cache", exc_info=True)
+            return
+
+        if cache_exists:
             logger.debug("Loading cached adblock data: %s", self._cache_path)
-            self._engine.deserialize_from_file(str(self._cache_path))
-        else:
-            if (
-                config.val.content.blocking.adblock.lists
-                and not self._has_basedir
-                and config.val.content.blocking.enabled
-                and self.enabled
-            ):
-                message.info("Run :adblock-update to get adblock lists.")
+            try:
+                with _map_exceptions():
+                    self._engine.deserialize_from_file(str(self._cache_path))
+            except DeserializationError:
+                message.error("Reading adblock filter data failed (corrupted data?). "
+                              "Please run :adblock-update.")
+        elif (
+            config.val.content.blocking.adblock.lists
+            and not self._has_basedir
+            and config.val.content.blocking.enabled
+            and self.enabled
+        ):
+            message.info("Run :adblock-update to get adblock lists.")
 
     def adblock_update(self) -> blockutils.BlocklistDownloads:
         """Update the adblock block lists."""
