@@ -21,7 +21,7 @@
 
 import os.path
 import contextlib
-from typing import TYPE_CHECKING, Iterator, List, Optional
+from typing import TYPE_CHECKING, Iterator, List, Optional, Any, Tuple
 
 from PyQt5.QtCore import QUrl
 
@@ -101,7 +101,7 @@ class ConfigCommands:
         Args:
             option: The name of the option.
             value: The value to set.
-            pattern: The URL pattern to use.
+            pattern: The link:configuring{outfilesuffix}#patterns[URL pattern] to use.
             temp: Set value temporarily until qutebrowser is closed.
             print_: Print the value after setting.
         """
@@ -151,9 +151,12 @@ class ConfigCommands:
             default: If given, restore a default binding.
         """
         if key is None:
+            url = QUrl('qute://bindings')
+            if mode != "normal":
+                url.setFragment(mode)
             tabbed_browser = objreg.get('tabbed-browser', scope='window',
                                         window=win_id)
-            tabbed_browser.load_url(QUrl('qute://bindings'), newtab=True)
+            tabbed_browser.load_url(url, newtab=True)
             return
 
         seq = self._parse_key(key)
@@ -204,8 +207,8 @@ class ConfigCommands:
 
         Args:
             option: The name of the option.
-            values: The values to cycle through.
-            pattern: The URL pattern to use.
+            *values: The values to cycle through.
+            pattern: The link:configuring{outfilesuffix}#patterns[URL pattern] to use.
             temp: Set value temporarily until qutebrowser is closed.
             print_: Print the value after setting.
         """
@@ -244,7 +247,14 @@ class ConfigCommands:
 
     @cmdutils.register(instance='config-commands')
     @cmdutils.argument('option', completion=configmodel.customized_option)
-    def config_unset(self, option: str, temp: bool = False) -> None:
+    @cmdutils.argument('pattern', flag='u')
+    def config_unset(
+        self,
+        option: str,
+        *,
+        pattern: str = None,
+        temp: bool = False,
+    ) -> None:
         """Unset an option.
 
         This sets an option back to its default and removes it from
@@ -252,10 +262,22 @@ class ConfigCommands:
 
         Args:
             option: The name of the option.
+            pattern: The link:configuring{outfilesuffix}#patterns[URL pattern] to use.
             temp: Set value temporarily until qutebrowser is closed.
         """
+        parsed_pattern = self._parse_pattern(pattern)
         with self._handle_config_error():
-            self._config.unset(option, save_yaml=not temp)
+            changed = self._config.unset(
+                option,
+                save_yaml=not temp,
+                pattern=parsed_pattern,
+            )
+
+        if not changed:
+            text = f'{option} is not customized'
+            if pattern is not None:
+                text += f' for {pattern}'
+            raise cmdutils.CommandError(text)
 
     @cmdutils.register(instance='config-commands')
     @cmdutils.argument('win_id', value=cmdutils.Value.win_id)
@@ -456,7 +478,7 @@ class ConfigCommands:
             raise cmdutils.CommandError("{} already exists - use --force to "
                                         "overwrite!".format(filename))
 
-        options: List = []
+        options: List[Tuple[Optional[urlmatch.UrlPattern], configdata.Option, Any]] = []
         if defaults:
             options = [(None, opt, opt.default)
                        for _name, opt in sorted(configdata.DATA.items())]
