@@ -173,7 +173,7 @@ class _BackendProblemChecker:
         """Show a dialog for a backend problem."""
         if self._no_err_windows:
             text = _error_text(*args, **kwargs)
-            print(text, file=sys.stderr)
+            log.init.error(text)
             sys.exit(usertypes.Exit.err_init)
 
         dialog = _Dialog(*args, **kwargs)
@@ -194,14 +194,6 @@ class _BackendProblemChecker:
 
         sys.exit(usertypes.Exit.err_init)
 
-    def _nvidia_shader_workaround(self) -> None:
-        """Work around QOpenGLShaderProgram issues.
-
-        See https://bugs.launchpad.net/ubuntu/+source/python-qt4/+bug/941826
-        """
-        self._assert_backend(usertypes.Backend.QtWebEngine)
-        utils.libgl_workaround()
-
     def _xwayland_options(self) -> Tuple[str, List[_Button]]:
         """Get buttons/text for a possible XWayland solution."""
         buttons = []
@@ -397,6 +389,7 @@ class _BackendProblemChecker:
         WORKAROUND for:
         https://bugreports.qt.io/browse/QTBUG-72532
         https://bugreports.qt.io/browse/QTBUG-82105
+        https://bugreports.qt.io/browse/QTBUG-93744
         """
         if ('serviceworker_workaround' not in configfiles.state['general'] and
                 qtutils.version_check('5.14', compiled=False)):
@@ -406,6 +399,8 @@ class _BackendProblemChecker:
             reason = 'Qt 5.14'
         elif configfiles.state.qt_version_changed:
             reason = 'Qt version changed'
+        elif configfiles.state.qtwe_version_changed:
+            reason = 'QtWebEngine version changed'
         elif config.val.qt.workarounds.remove_service_workers:
             reason = 'Explicitly enabled'
         else:
@@ -435,7 +430,6 @@ class _BackendProblemChecker:
         self._check_backend_modules()
         if objects.backend == usertypes.Backend.QtWebEngine:
             self._handle_ssl_support()
-            self._nvidia_shader_workaround()
             self._handle_wayland_webgl()
             self._handle_cache_nuking()
             self._handle_serviceworker_nuking()
