@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2015-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2015-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,9 +15,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
-
-import json
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 import pytest_bdd as bdd
 bdd.scenarios('misc.feature')
@@ -28,10 +26,3 @@ def pdf_exists(quteproc, tmpdir, filename):
     path = tmpdir / filename
     data = path.read_binary()
     assert data.startswith(b'%PDF')
-
-
-@bdd.when(bdd.parsers.parse('I set up "{lists}" as block lists'))
-def set_up_blocking(quteproc, lists, server):
-    url = 'http://localhost:{}/data/adblock/'.format(server.port)
-    urls = [url + item.strip() for item in lists.split(',')]
-    quteproc.set_setting('content.host_blocking.lists', json.dumps(urls))
