@@ -26,6 +26,7 @@ import logging
 import importlib
 import re
 import json
+import platform
 
 import pytest
 from PyQt5.QtCore import QProcess, QPoint
@@ -39,6 +40,14 @@ ascii_locale = pytest.mark.skipif(sys.hexversion >= 0x03070000,
                                   "locale with LC_ALL=C")
 
 
+# For some reason (some floating point rounding differences?), color values are
+# slightly different (and wrong!) on ARM machines. We adjust our expected values
+# accordingly, since we don't really care about the exact value, we just want to
+# know that the underlying Chromium is respecting our preferences.
+# FIXME what to do about 32-bit ARM?
+IS_ARM = platform.machine() == 'aarch64'
+
+
 def _base_args(config):
     """Get the arguments to pass with every invocation."""
     args = ['--debug', '--json-logging', '--no-err-windows']
@@ -47,15 +56,24 @@ def _base_args(config):
     else:
         args += ['--backend', 'webkit']
 
-    if config.webengine:
-        args += testutils.seccomp_args(qt_flag=True)
+    if config.webengine and testutils.disable_seccomp_bpf_sandbox():
+        args += testutils.DISABLE_SECCOMP_BPF_ARGS
 
     args.append('about:blank')
     return args
 
 
 @pytest.fixture
-def temp_basedir_env(tmp_path, short_tmpdir):
+def runtime_tmpdir(short_tmpdir):
+    """A directory suitable for XDG_RUNTIME_DIR."""
+    runtime_dir = short_tmpdir / 'rt'
+    runtime_dir.ensure(dir=True)
+    runtime_dir.chmod(0o700)
+    return runtime_dir
+
+
+@pytest.fixture
+def temp_basedir_env(tmp_path, runtime_tmpdir):
     """Return a dict of environment variables that fakes --temp-basedir.
 
     We can't run --basedir or --temp-basedir for some tests, so we mess with
@@ -63,12 +81,8 @@ def temp_basedir_env(tmp_path, short_tmpdir):
     """
     data_dir = tmp_path / 'data'
     config_dir = tmp_path / 'config'
-    runtime_dir = short_tmpdir / 'rt'
     cache_dir = tmp_path / 'cache'
 
-    runtime_dir.ensure(dir=True)
-    runtime_dir.chmod(0o700)
-
     lines = [
         '[general]',
         'quickstart-done = 1',
@@ -83,7 +97,7 @@ def temp_basedir_env(tmp_path, short_tmpdir):
     env = {
         'XDG_DATA_HOME': str(data_dir),
         'XDG_CONFIG_HOME': str(config_dir),
-        'XDG_RUNTIME_DIR': str(runtime_dir),
+        'XDG_RUNTIME_DIR': str(runtime_tmpdir),
         'XDG_CACHE_HOME': str(cache_dir),
     }
     return env
@@ -512,7 +526,8 @@ def test_preferred_colorscheme_with_dark_mode(
         # No workaround known.
         expected_text = 'Light preference detected.'
         # light website color, inverted by darkmode
-        expected_color = testutils.Color(127, 127, 127)
+        expected_color = (testutils.Color(123, 125, 123) if IS_ARM
+                          else testutils.Color(127, 127, 127))
         xfail = "Chromium bug 1177973"
     elif qtwe_version == utils.VersionNumber(5, 15, 2):
         # Our workaround breaks when dark mode is enabled...
@@ -524,7 +539,8 @@ def test_preferred_colorscheme_with_dark_mode(
         # Qt 5.14 and 5.15.0/.1 work correctly.
         # Hopefully, so does Qt 6.x in the future?
         expected_text = 'Dark preference detected.'
-        expected_color = testutils.Color(34, 34, 34)  # dark website color
+        expected_color = (testutils.Color(33, 32, 33) if IS_ARM
+                          else testutils.Color(34, 34, 34))  # dark website color
         xfail = False
 
     pos = QPoint(0, 0)
@@ -625,30 +641,51 @@ def test_cookies_store(quteproc_new, request, short_tmpdir, store):
     quteproc_new.wait_for_quit()
 
 
+# The 'colors' dictionaries in the parametrize decorator below have (QtWebEngine
+# version, CPU architecture) as keys. Either of those (or both) can be None to
+# say "on all other Qt versions" or "on all other CPU architectures".
 @pytest.mark.parametrize('filename, algorithm, colors', [
     (
         'blank',
         'lightness-cielab',
         {
-            '5.15': testutils.Color(18, 18, 18),
-            '5.14': testutils.Color(27, 27, 27),
-            None: testutils.Color(0, 0, 0),
+            ('5.15', None): testutils.Color(18, 18, 18),
+            ('5.15', 'aarch64'): testutils.Color(16, 16, 16),
+            ('5.14', None): testutils.Color(27, 27, 27),
+            ('5.14', 'aarch64'): testutils.Color(24, 24, 24),
+            (None, None): testutils.Color(0, 0, 0),
         }
     ),
-    ('blank', 'lightness-hsl', {None: testutils.Color(0, 0, 0)}),
-    ('blank', 'brightness-rgb', {None: testutils.Color(0, 0, 0)}),
+    ('blank', 'lightness-hsl', {(None, None): testutils.Color(0, 0, 0)}),
+    ('blank', 'brightness-rgb', {(None, None): testutils.Color(0, 0, 0)}),
 
     (
         'yellow',
         'lightness-cielab',
         {
-            '5.15': testutils.Color(35, 34, 0),
-            '5.14': testutils.Color(35, 34, 0),
-            None: testutils.Color(204, 204, 0),
+            ('5.15', None): testutils.Color(35, 34, 0),
+            ('5.15', 'aarch64'): testutils.Color(33, 32, 0),
+            ('5.14', None): testutils.Color(35, 34, 0),
+            ('5.14', 'aarch64'): testutils.Color(33, 32, 0),
+            (None, None): testutils.Color(204, 204, 0),
+        }
+    ),
+    (
+        'yellow',
+        'lightness-hsl',
+        {
+            (None, None): testutils.Color(204, 204, 0),
+            (None, 'aarch64'): testutils.Color(206, 207, 0),
+        },
+    ),
+    (
+        'yellow',
+        'brightness-rgb',
+        {
+            (None, None): testutils.Color(0, 0, 204),
+            (None, 'aarch64'): testutils.Color(0, 0, 206),
         }
     ),
-    ('yellow', 'lightness-hsl', {None: testutils.Color(204, 204, 0)}),
-    ('yellow', 'brightness-rgb', {None: testutils.Color(0, 0, 204)}),
 ])
 def test_dark_mode(webengine_versions, quteproc_new, request,
                    filename, algorithm, colors):
@@ -664,7 +701,17 @@ def test_dark_mode(webengine_versions, quteproc_new, request,
 
     ver = webengine_versions.webengine
     minor_version = str(ver.strip_patch())
-    expected = colors.get(minor_version, colors[None])
+
+    arch = platform.machine()
+    for key in [
+        (minor_version, arch),
+        (minor_version, None),
+        (None, arch),
+        (None, None),
+    ]:
+        if key in colors:
+            expected = colors[key]
+            break
 
     quteproc_new.open_path(f'data/darkmode/{filename}.html')
 
@@ -691,9 +738,11 @@ def test_dark_mode_mathml(quteproc_new, request, qtbot):
     quteproc_new.wait_for_js('Image loaded')
 
     # First make sure loading finished by looking outside of the image
+    expected = testutils.Color(0, 0, 206) if IS_ARM else testutils.Color(0, 0, 204)
+
     quteproc_new.get_screenshot(
         probe_pos=QPoint(105, 0),
-        probe_color=testutils.Color(0, 0, 204),
+        probe_color=expected,
     )
 
     # Then get the actual formula color, probing again in case it's not displayed yet...
@@ -703,6 +752,30 @@ def test_dark_mode_mathml(quteproc_new, request, qtbot):
     )
 
 
+@testutils.qt514
+@pytest.mark.parametrize('value, preference', [
+    ('true', 'Reduced motion'),
+    ('false', 'No'),
+])
+@pytest.mark.skipif(
+    utils.is_windows,
+    reason="Outcome on Windows depends on system settings",
+)
+def test_prefers_reduced_motion(quteproc_new, request, value, preference):
+    if not request.config.webengine:
+        pytest.skip("Skipped with QtWebKit")
+
+    args = _base_args(request.config) + [
+        '--temp-basedir',
+        '-s', 'content.prefers_reduced_motion', value,
+    ]
+    quteproc_new.start(args)
+
+    quteproc_new.open_path('data/prefers_reduced_motion.html')
+    content = quteproc_new.get_content()
+    assert content == f"{preference} preference detected."
+
+
 def test_unavailable_backend(request, quteproc_new):
     """Test starting with a backend which isn't available.
 
@@ -743,3 +816,91 @@ def test_unavailable_backend(request, quteproc_new):
         message=('*qutebrowser tried to start with the Qt* backend but failed '
                  'because * could not be imported.*'))
     line.expected = True
+
+
+def test_json_logging_without_debug(request, quteproc_new, runtime_tmpdir):
+    args = _base_args(request.config) + ['--temp-basedir', ':quit']
+    args.remove('--debug')
+    args.remove('about:blank')  # interfers with :quit at the end
+
+    quteproc_new.exit_expected = True
+    quteproc_new.start(args, env={'XDG_RUNTIME_DIR': str(runtime_tmpdir)})
+    assert not quteproc_new.is_running()
+
+
+@pytest.mark.qtwebkit_skip
+@pytest.mark.parametrize(
+    'sandboxing, has_namespaces, has_seccomp, has_yama, expected_result', [
+        ('enable-all', True, True, True, "You are adequately sandboxed."),
+        ('disable-seccomp-bpf', True, False, True, "You are NOT adequately sandboxed."),
+        ('disable-all', False, False, False, "You are NOT adequately sandboxed."),
+    ]
+)
+def test_sandboxing(
+        request, quteproc_new, sandboxing,
+        has_namespaces, has_seccomp, has_yama, expected_result,
+):
+    if not request.config.webengine:
+        pytest.skip("Skipped with QtWebKit")
+    elif sandboxing == "enable-all" and testutils.disable_seccomp_bpf_sandbox():
+        pytest.skip("Full sandboxing not supported")
+
+    args = _base_args(request.config) + [
+        '--temp-basedir',
+        '-s', 'qt.chromium.sandboxing', sandboxing,
+    ]
+    quteproc_new.start(args)
+
+    quteproc_new.open_url('chrome://sandbox')
+    text = quteproc_new.get_content()
+    print(text)
+
+    not_found_msg = ("The webpage at chrome://sandbox/ might be temporarily down or "
+                     "it may have moved permanently to a new web address.")
+    if not_found_msg in text.split("\n"):
+        line = quteproc_new.wait_for(message='Load error: ERR_INVALID_URL')
+        line.expected = True
+        pytest.skip("chrome://sandbox/ not supported")
+
+    bpf_text = "Seccomp-BPF sandbox"
+    yama_text = "Ptrace Protection with Yama LSM"
+
+    if "\n\n\n" in text:
+        # Qt 5.12
+        header, rest = text.split("\n", maxsplit=1)
+        rest, result = rest.rsplit("\n\n", maxsplit=1)
+        lines = rest.replace("\t\n", "\t").split("\n\n\n")
+
+        expected_status = {
+            "Namespace Sandbox": "Yes" if has_namespaces else "No",
+            "Network namespaces": "Yes" if has_namespaces else "No",
+            "PID namespaces": "Yes" if has_namespaces else "No",
+            "SUID Sandbox": "No",
+
+            bpf_text: "Yes" if has_seccomp else "No",
+            f"{bpf_text} supports TSYNC": "Yes" if has_seccomp else "No",
+
+            "Yama LSM Enforcing": "Yes" if has_yama else "No",
+        }
+    else:
+        header, *lines, empty, result = text.split("\n")
+        assert not empty
+
+        expected_status = {
+            "Layer 1 Sandbox": "Namespace" if has_namespaces else "None",
+
+            "PID namespaces": "Yes" if has_namespaces else "No",
+            "Network namespaces": "Yes" if has_namespaces else "No",
+
+            bpf_text: "Yes" if has_seccomp else "No",
+            f"{bpf_text} supports TSYNC": "Yes" if has_seccomp else "No",
+
+            f"{yama_text} (Broker)": "Yes" if has_yama else "No",
+            f"{yama_text} (Non-broker)": "No",
+        }
+
+    assert header == "Sandbox Status"
+    assert result == expected_result
+
+    status = dict(line.split("\t") for line in lines)
+    assert status == expected_status
