@@ -27,7 +27,7 @@ from PyQt5.QtCore import QUrl
 
 from qutebrowser.config import configcommands
 from qutebrowser.api import cmdutils
-from qutebrowser.utils import usertypes, urlmatch
+from qutebrowser.utils import usertypes, urlmatch, utils
 from qutebrowser.keyinput import keyutils
 from qutebrowser.misc import objects
 
@@ -460,6 +460,40 @@ class TestUnsetAndClear:
         with pytest.raises(cmdutils.CommandError, match="No option 'tabs'"):
             commands.config_unset('tabs')
 
+    def test_unset_uncustomized(self, commands):
+        with pytest.raises(cmdutils.CommandError, match="tabs.show is not customized"):
+            commands.config_unset('tabs.show')
+
+    @pytest.mark.parametrize('set_global', [True, False])
+    def test_unset_pattern(self, commands, config_stub, set_global):
+        name = 'content.javascript.enabled'
+        pattern = urlmatch.UrlPattern('*://example.com')
+        url = QUrl('https://example.com')
+
+        if set_global:
+            config_stub.set_obj(name, False)
+            global_value = False
+            local_value = True
+        else:
+            global_value = True
+            local_value = False
+
+        config_stub.set_obj(name, local_value, pattern=pattern)
+        commands.config_unset(name, pattern=str(pattern))
+        assert config_stub.get_obj(name, url=url) == global_value
+        assert config_stub.get_obj(name, url=url, fallback=False) == usertypes.UNSET
+
+    def test_unset_uncustomized_pattern(self, commands, config_stub):
+        name = 'content.javascript.enabled'
+        pattern = 'example.com'
+
+        config_stub.set_obj(name, False)
+        with pytest.raises(
+            cmdutils.CommandError,
+            match=f"{name} is not customized for {pattern}",
+        ):
+            commands.config_unset(name, pattern=pattern)
+
     @pytest.mark.parametrize('save', [True, False])
     def test_clear(self, commands, config_stub, yaml_value, save):
         name = 'tabs.show'
@@ -480,7 +514,7 @@ class TestSource:
 
     @pytest.mark.parametrize('location', ['default', 'absolute', 'relative'])
     @pytest.mark.parametrize('clear', [True, False])
-    def test_config_source(self, tmpdir, commands, config_stub, config_tmpdir,
+    def test_config_source(self, tmp_path, commands, config_stub, config_tmpdir,
                            location, clear):
         assert config_stub.val.content.javascript.enabled
         config_stub.val.search.ignore_case = 'always'
@@ -489,13 +523,13 @@ class TestSource:
             pyfile = config_tmpdir / 'config.py'
             arg = None
         elif location == 'absolute':
-            pyfile = tmpdir / 'sourced.py'
+            pyfile = tmp_path / 'sourced.py'
             arg = str(pyfile)
         elif location == 'relative':
             pyfile = config_tmpdir / 'sourced.py'
             arg = 'sourced.py'
         else:
-            assert False, location
+            raise utils.Unreachable(location)
 
         pyfile.write_text('\n'.join(['config.load_autoconfig(False)',
                                      'c.content.javascript.enabled = False']),
@@ -607,8 +641,8 @@ class TestWritePy:
 
     """Tests for :config-write-py."""
 
-    def test_custom(self, commands, config_stub, key_config_stub, tmpdir):
-        confpy = tmpdir / 'config.py'
+    def test_custom(self, commands, config_stub, key_config_stub, tmp_path):
+        confpy = tmp_path / 'config.py'
         config_stub.val.content.javascript.enabled = True
         key_config_stub.bind(keyseq(',x'), 'message-info foo', mode='normal')
 
@@ -618,8 +652,8 @@ class TestWritePy:
         assert "c.content.javascript.enabled = True" in lines
         assert "config.bind(',x', 'message-info foo')" in lines
 
-    def test_defaults(self, commands, tmpdir):
-        confpy = tmpdir / 'config.py'
+    def test_defaults(self, commands, tmp_path):
+        confpy = tmp_path / 'config.py'
         commands.config_write_py(str(confpy), defaults=True)
 
         lines = confpy.read_text('utf-8').splitlines()
@@ -639,10 +673,10 @@ class TestWritePy:
         assert '# Autogenerated config.py' in lines
 
     @pytest.mark.posix
-    def test_expanduser(self, commands, monkeypatch, tmpdir):
+    def test_expanduser(self, commands, monkeypatch, tmp_path):
         """Make sure that using a path with ~/... works correctly."""
-        home = tmpdir / 'home'
-        home.ensure(dir=True)
+        home = tmp_path / 'home'
+        home.mkdir()
         monkeypatch.setenv('HOME', str(home))
 
         commands.config_write_py('~/config.py')
@@ -651,9 +685,9 @@ class TestWritePy:
         lines = confpy.read_text('utf-8').splitlines()
         assert '# Autogenerated config.py' in lines
 
-    def test_existing_file(self, commands, tmpdir):
-        confpy = tmpdir / 'config.py'
-        confpy.ensure()
+    def test_existing_file(self, commands, tmp_path):
+        confpy = tmp_path / 'config.py'
+        confpy.touch()
 
         with pytest.raises(cmdutils.CommandError) as excinfo:
             commands.config_write_py(str(confpy))
@@ -661,19 +695,19 @@ class TestWritePy:
         expected = " already exists - use --force to overwrite!"
         assert str(excinfo.value).endswith(expected)
 
-    def test_existing_file_force(self, commands, tmpdir):
-        confpy = tmpdir / 'config.py'
-        confpy.ensure()
+    def test_existing_file_force(self, commands, tmp_path):
+        confpy = tmp_path / 'config.py'
+        confpy.touch()
 
         commands.config_write_py(str(confpy), force=True)
 
         lines = confpy.read_text('utf-8').splitlines()
         assert '# Autogenerated config.py' in lines
 
-    def test_oserror(self, commands, tmpdir):
+    def test_oserror(self, commands, tmp_path):
         """Test writing to a directory which does not exist."""
         with pytest.raises(cmdutils.CommandError):
-            commands.config_write_py(str(tmpdir / 'foo' / 'config.py'))
+            commands.config_write_py(str(tmp_path / 'foo' / 'config.py'))
 
     def test_config_py_arg(self, commands, config_py_arg):
         config_py_arg.ensure()
@@ -691,15 +725,19 @@ class TestBind:
         """Get a dict with no bindings."""
         return {'normal': {}}
 
+    @pytest.mark.parametrize("mode, url", [
+        ("normal", QUrl("qute://bindings")),
+        ("passthrough", QUrl("qute://bindings#passthrough")),
+    ])
     def test_bind_no_args(self, commands, config_stub, no_bindings,
-                          tabbed_browser_stubs):
+                          tabbed_browser_stubs, mode, url):
         """Run ':bind'.
 
         Should open qute://bindings."""
         config_stub.val.bindings.default = no_bindings
         config_stub.val.bindings.commands = no_bindings
-        commands.bind(win_id=0)
-        assert tabbed_browser_stubs[0].loaded_url == QUrl('qute://bindings')
+        commands.bind(win_id=0, mode=mode)
+        assert tabbed_browser_stubs[0].loaded_url == url
 
     @pytest.mark.parametrize('command', ['nop', 'nope'])
     def test_bind(self, commands, config_stub, no_bindings, key_config_stub,
