@@ -1,4 +1,4 @@
-# Copyright 2015-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2015-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
 # This file is part of qutebrowser.
@@ -14,7 +14,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Tests for qutebrowser.misc.checkpyver."""
 
@@ -28,21 +28,22 @@ import pytest
 from qutebrowser.misc import checkpyver
 
 
-TEXT = (r"At least Python 3.5.2 is required to run qutebrowser, but it's "
+TEXT = (r"At least Python 3.6.1 is required to run qutebrowser, but it's "
         r"running with \d+\.\d+\.\d+.")
 
 
 @pytest.mark.not_frozen
-def test_python2():
-    """Run checkpyver with python 2."""
+@pytest.mark.parametrize('python', ['python2', 'python3.5'])
+def test_old_python(python):
+    """Run checkpyver with old python versions."""
     try:
         proc = subprocess.run(
-            ['python2', checkpyver.__file__, '--no-err-windows'],
+            [python, checkpyver.__file__, '--no-err-windows'],
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
             check=False)
     except FileNotFoundError:
-        pytest.skip("python2 not found")
+        pytest.skip(f"{python} not found")
     assert not proc.stdout
     stderr = proc.stderr.decode('utf-8').rstrip()
     assert re.fullmatch(TEXT, stderr), stderr
