@@ -334,7 +334,6 @@ def test_get_search_url_open_base_url(config_stub, url, host):
     Args:
         url: The "URL" to enter.
         host: The expected search machine host.
-        query: The expected search query.
     """
     config_stub.val.url.open_base_url = True
     url = urlutils._get_search_url(url)
@@ -778,3 +777,26 @@ class TestParseJavascriptUrl:
             pass
         else:
             assert parsed == source
+
+
+class TestWiden:
+
+    @pytest.mark.parametrize('hostname, expected', [
+        ('a.b.c', ['a.b.c', 'b.c', 'c']),
+        ('foobarbaz', ['foobarbaz']),
+        ('', []),
+        ('.c', ['.c', 'c']),
+        ('c.', ['c.']),
+        ('.c.', ['.c.', 'c.']),
+        (None, []),
+    ])
+    def test_widen_hostnames(self, hostname, expected):
+        assert list(urlutils.widened_hostnames(hostname)) == expected
+
+    @pytest.mark.parametrize('hostname', [
+        'test.qutebrowser.org',
+        'a.b.c.d.e.f.g.h.i.j.k.l.m.n.o.p.q.r.s.t.u.v.w.z.y.z',
+        'qqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqq.c',
+    ])
+    def test_bench_widen_hostnames(self, hostname, benchmark):
+        benchmark(lambda: list(urlutils.widened_hostnames(hostname)))
